'use strict';

Object.defineProperty(exports, '__esModule', {
  value: true
});
exports.readFile = readFile;
exports.read = read;

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

var _recast = require('recast');

var _recast2 = _interopRequireDefault(_recast);

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _coffeeScript = require('coffee-script');

var _coffeeScript2 = _interopRequireDefault(_coffeeScript);

/**
 * This function reads a js file and transforms it into AST
 *
 * @author Mohamad Mohebifar
 * @param file
 * @param options
 * @returns {Object}
 */

function readFile(file, options) {
  var _this = this;

  if (typeof options.coffee === 'undefined') {
    options.coffee = /\.coffee$/.test(file);
  }

  if (options.sync) {
    var js = _fs2['default'].readFileSync(file);

    return this.read(js, options);
  } else {
    _fs2['default'].readFile(file, function (js) {
      if (options.callback) {
        options.callback(_this.read(js, options));
      }
    });
  }
}

/**
 * This function reads a js string and transforms it into AST
 *
 * @author Mohamad Mohebifar
 * @param js
 * @param options
 * @returns {Object}
 */

function read(js, options) {
  if (options.coffee) {
    js = _coffeeScript2['default'].compile(js);
  }

  var ast = _recast2['default'].parse(js).program;

  return ast;
}

exports['default'] = {
  read: read,
  readFile: readFile
};